package main

import (
	"fmt"
	"sync"
	"encoding/json"
	"sync/atomic"
	"github.com/aliyun/alibaba-cloud-sdk-go/services/geoip"
	"q5"
	"f5"
)

type RiskMgr struct {
	totalPassTimes int64
	totalBlockTimes int64
	passTimes int64
	blockTimes int64

	totalHttpMethodBlockTimes int64
	httpMethodBlockTimes int64
	totalSessionErrBlockTimes int64
	sessionErrBlockTimes int64

	gameHash map[int64]*GameConf
	gameHashMutex sync.RWMutex

	clusterConf *MtwServerInfo
}

func (this* RiskMgr) Init() *RiskMgr {
	this.gameHashMutex.Lock()
	defer this.gameHashMutex.Unlock()

	this.gameHash = make(map[int64]*GameConf)
	this.clusterConf = G.MetaMgr.GetCurrServer()

	G.HttpServer.RegisterHandle("Analyse", "isOpen", this._analyseIsOpen)
	G.HttpServer.RegisterHandle("Ops", "syncConf", this._opsSyncConf)
	G.HttpServer.RegisterHandle("Ops", "info", this._opsInfo)
	f5.Timer().AddRepeatTimer(1000 * 60,
		func (params* q5.XParams) {
		},
		func (params* q5.XParams) {
			this.OutputLog()
		})
	f5.App.RegisterIMMsgHandle(IM_GAMECONF_CREATE, func (msgId int16, params *q5.XParams) {
		G.RiskMgr.OnGameConfCreate(
			params.Sender.GetInt32(),
			params.Param1.GetInt32())
	})
	return this
}

func (this* RiskMgr) UnInit() {
}

func (this* RiskMgr) fetchGameConfList() []*GameConf {
	this.gameHashMutex.Lock()
	defer this.gameHashMutex.Unlock()

	gameConfList := make([]*GameConf, 0)
	for _, val := range this.gameHash {
		gameConfList = append(gameConfList, val)
	}
	return gameConfList
}

func (this* RiskMgr) OutputLog() {
	for _, val := range this.fetchGameConfList() {
		f5.SysLog().Info("gameid:%d channel:%d passobj %s",
			val.gameId,
			val.channel,
			val.GetPassObj().ToJsonStr())
		f5.SysLog().Info("gameid:%d channel:%d blockobj %s",
			val.gameId,
			val.channel,
			val.GetBlockObj().ToJsonStr())
	}
	f5.SysLog().Info("global passobj %s", this.GetGlobalPassObj().ToJsonStr())
	f5.SysLog().Info("global blockobj %s", this.GetGlobalBlockObj().ToJsonStr())
	atomic.StoreInt64(&this.passTimes, 0)
	atomic.StoreInt64(&this.blockTimes, 0)
	atomic.StoreInt64(&this.httpMethodBlockTimes, 0)
	atomic.StoreInt64(&this.sessionErrBlockTimes, 0)
}

func (this* RiskMgr) GetGlobalPassObj() *q5.XObject {
	passObj := q5.NewMxoObject()
	passObj.SetXValue("total", q5.NewXInt64(this.totalPassTimes))
	passObj.SetXValue("curr", q5.NewXInt64(this.passTimes))
	return passObj.AsXObject()
}

func (this* RiskMgr) GetGlobalBlockObj() *q5.XObject {
	blockObj := q5.NewMxoObject()
	blockObj.SetXValue("total", q5.NewXInt64(this.totalBlockTimes))
	blockObj.SetXValue("curr", q5.NewXInt64(this.blockTimes))
	blockObj.SetXValue("total_methoderr", q5.NewXInt64(this.totalHttpMethodBlockTimes))
	blockObj.SetXValue("curr_methoderr", q5.NewXInt64(this.httpMethodBlockTimes))
	blockObj.SetXValue("total_sessonerr", q5.NewXInt64(this.totalSessionErrBlockTimes))
	blockObj.SetXValue("curr_sessionerr", q5.NewXInt64(this.sessionErrBlockTimes))
	return blockObj.AsXObject()
}

func (this *RiskMgr) ForceGameConf(gameId int32, channel int32) *GameConf {
	this.gameHashMutex.Lock()
	defer this.gameHashMutex.Unlock()
	if val, ok := this.gameHash[q5.MkInt64(gameId, channel)]; ok {
		val.Active()
		return val
	} else {
		gameConf := new(GameConf)
		gameConf.gameId = gameId
		gameConf.channel = channel
		gameConf.Init()
		this.gameHash[q5.MkInt64(gameId, channel)] = gameConf
		gameConf.Active()
		return  gameConf
	}
}

func (this *RiskMgr) GetGameConf(gameId int32, channel int32) *GameConf {
	this.gameHashMutex.Lock()
	defer this.gameHashMutex.Unlock()
	val, _ := this.gameHash[q5.MkInt64(gameId, channel)]
	return  val
}

func (this *RiskMgr) _analyseIsOpen(c *f5.Context) {
	var gameConf *GameConf
	responseStr := ""
	launchInfo := ""
	isPass := func () bool {
		if c.Method() != "POST" {
			atomic.AddInt64(&this.totalHttpMethodBlockTimes, 1)
			atomic.AddInt64(&this.httpMethodBlockTimes, 1)
			return false
		}
		accountId := c.Request("account_id").GetString()
		sessionId := c.Request("session_id").GetString()
		if !f5.IsValidSessionId(accountId, sessionId, SESSION_KEY) {
			atomic.AddInt64(&this.totalSessionErrBlockTimes, 1)
			atomic.AddInt64(&this.sessionErrBlockTimes, 1)
			return false
		}
		gameId := f5.ExtractGameIdFromAccountId(accountId)
		channel := f5.ExtractChannelFromAccountId(accountId)

		gameConf = this.ForceGameConf(gameId, channel)
		if gameConf == nil {
			return false
		}

		remoteAddr := c.GetRemoteAddr()
		if this.clusterConf.GetTesting() == 1 && c.Request("ip").GetString() != "" {
			remoteAddr = c.Request("ip").GetString()
		}

		launchInfo = c.GetBody().GetString()
		if gameConf.IsPass(remoteAddr, launchInfo, &responseStr) {
			atomic.AddInt64(&gameConf.passTimes, 1)
			atomic.AddInt64(&gameConf.totalPassTimes, 1)
			return true
		} else {
			atomic.AddInt64(&gameConf.blockTimes, 1)
			atomic.AddInt64(&gameConf.totalBlockTimes, 1)
			return false
		}
	}
	beginTick := q5.GetTickCount()
	if isPass() {
		atomic.AddInt64(&this.totalPassTimes, 1)
		atomic.AddInt64(&this.passTimes, 1)
		if this.clusterConf.GetTesting() == 1 {
			if responseStr == "" {
				responseStr = `""`
			}
			data := fmt.Sprintf(`{"errcode":0, "errmsg":"", "is_open":%d, "data":%s}`,
				1,
				responseStr)
			c.Response(data)
		} else {
			c.ResponseInt32Ok("is_open", 1)
		}
		f5.SysLog().Info("time:%d analyse pass:%s launch:%s",
			q5.GetTickCount() - beginTick,
			responseStr,
			launchInfo)
	} else {
		atomic.AddInt64(&this.totalBlockTimes, 1)
		atomic.AddInt64(&this.blockTimes, 1)
		if this.clusterConf.GetTesting() == 1 {
			if responseStr == "" {
				responseStr = `""`
			}
			data := fmt.Sprintf(`{"errcode":0, "errmsg":"", "is_open":%d, "data":%s}`,
				0,
				responseStr)
			c.Response(data)
		} else {
			c.ResponseInt32Ok("is_open", 0)
		}
		f5.SysLog().Warning("time:%d analyse block:%s launch:%s",
			q5.GetTickCount() - beginTick,
			responseStr,
			launchInfo)
	}
}

func (this *RiskMgr) _opsSyncConf(c *f5.Context) {
	gameId := c.Request("gameid").GetInt32()
	channel := c.Request("channel").GetInt32()
	gameConf := this.GetGameConf(gameId, channel)
	if gameConf == nil {
		c.ResponseErr(1, "数据不存在")
		return
	}
	gameConf.Active()
	gameConf.SyncConf()
	c.ResponseOk()
}

func (this *RiskMgr) _opsInfo(c *f5.Context) {
	respObj := q5.NewMxoObject()
	respObj.SetXValue("errcode", q5.NewXInt32(0))
	respObj.SetXValue("errmsg", q5.NewXString(""))
	respObj.SetXObject("global_pass", this.GetGlobalPassObj())
	respObj.SetXObject("global_block", this.GetGlobalBlockObj())
	gameList := q5.NewMxoArray()
	for _, gameConf := range this.fetchGameConfList() {
		gameConf.Active()
		gameObj := q5.NewMxoObject()
		gameObj.SetXValue("gameid", q5.NewXInt32(gameConf.gameId))
		gameObj.SetXValue("channel", q5.NewXInt32(gameConf.channel))
		gameObj.SetXObject("conf", gameConf.GetConfObj())
		gameObj.SetXObject("pass", gameConf.GetPassObj())
		gameObj.SetXObject("block", gameConf.GetBlockObj())
		gameList.PushXObject(gameObj.AsXObject())
	}
	respObj.SetXObject("games", gameList.AsXObject())
	c.Response(respObj.ToJsonStr())
}

/*
安装定时器定时更新逻辑
*/
func (this *RiskMgr) OnGameConfCreate(gameId int32, channel int32) {
	f5.Timer().AddRepeatTimer(1000 * 30,
		func (params* q5.XParams) {
		},
		func (params* q5.XParams) {
			gameConf := this.GetGameConf(gameId, channel)
			if gameConf != nil {
				gameConf.SyncConf()
			}
		})
}

func (this* RiskMgr) IsSafeZone(ip string, responseStr* string) bool {
	country := ""
	province := ""
	city := ""
	if this.GetIpInfo(ip, responseStr, &country, &province, &city) {
		return !G.MetaMgr.IsBlockZone(country, province, city)
	} else {
		return false;
	}
}

func (this* RiskMgr) GetIpInfo(
	ip string,
	responseStr* string,
	country* string,
	province* string,
	city* string) bool {
	return this.GetIpInfoByTx(ip, responseStr, country, province, city)
}

func (this* RiskMgr) GetIpInfoByAli(
	ip string,
	responseStr* string,
	country* string,
	province* string,
	city* string) bool {
	accessKeyId := ""
	accessSecret := ""
	G.MetaMgr.GetAliSdkInfo(&accessKeyId, &accessSecret)
	client, err := geoip.NewClientWithAccessKey("cn-hangzhou", accessKeyId, accessSecret)
	if err != nil {
		f5.SysLog().Warning("NewClientWithAccessKey error %s", err.Error())
		return false
	}

	request := geoip.CreateDescribeIpv4LocationRequest()
	request.Scheme = "https"
	request.Ip = ip

	response, err := client.DescribeIpv4Location(request)
	if err != nil {
		f5.SysLog().Warning("Ipv4Location error %s", err.Error())
		return false
	}
	if response != nil {
		f5.SysLog().Info("ali_response: ", response)
	}
	*responseStr = q5.EncodeJson(response)
	*country = response.Country
	*province = response.Province
	*city = response.City
	return true
}

func (this* RiskMgr) GetIpInfoByTx(
	ip string,
	responseStr* string,
	country* string,
	province* string,
	city* string) bool {
	txKey := ""
	txSK := ""
	G.MetaMgr.GetTxSdkInfo(&txKey, &txSK)
	url := "https://apis.map.qq.com/ws/location/v1/ip"
	sign := q5.Md5Str("/ws/location/v1/ip?" + "ip=" + ip + "&key=" + txKey + txSK)
	params := q5.NewMxoObject()
	params.SetXValue("ip", q5.NewXString(ip))
	params.SetXValue("key", q5.NewXString(txKey))
	params.SetXValue("sig", q5.NewXString(sign))
	_, respStr, err := q5.HttpGetAsJson(
		url,
		params.AsXObject())
	if err != nil {
		f5.SysLog().Warning("GetIpInfoByTx error %s", err.Error())
		return false
	}
	*responseStr = respStr
	respObj := struct {
		Status *int32 `json:"status"`
		Result *struct {
			AdInfo *struct {
				Nation string `json:"nation"`
				Province string `json:"province"`
				City string `json:"city"`
			} `json:"ad_info"`
		} `json:"result"`
	}{}
	if err := json.Unmarshal([]byte(respStr), &respObj); err != nil || respObj.Status == nil {
		f5.SysLog().Warning("GetIpInfoByTx response1 %s", respStr)
		return false
	}
	if respObj.Result == nil {
		f5.SysLog().Warning("GetIpInfoByTx response2 %s", respStr)
		return false
	}
	if respObj.Result.AdInfo == nil {
		f5.SysLog().Warning("GetIpInfoByTx response3 %s", respStr)
		return false
	}
	*country = respObj.Result.AdInfo.Nation
	*province = respObj.Result.AdInfo.Province
	*city = respObj.Result.AdInfo.City
	return true
}
