package main

import (
	"sync"
	"sync/atomic"
	"fmt"
	"q5"
	"f5"
)

type Game2005 struct {
	fetchEventAtomic int64
	dailyMailAtomic int64
	mysqlCluster []*MtwGame2005MysqlConf
	mysqlClusterMutex sync.RWMutex
	dbIdxHash sync.Map
	vipLvUpMailMeta *MtwGame2005Mail
	priviDailyMailMeta *MtwGame2005Mail
	vipWeeklyMailMeta *MtwGame2005Mail
}

func (this *Game2005) Init() {
	this.mysqlCluster = G.MetaMgr.GetGame2005MysqlCluster()
	this.vipLvUpMailMeta = G.MetaMgr.GetGame2005Mail(1)
	this.priviDailyMailMeta = G.MetaMgr.GetGame2005Mail(2)
	this.vipWeeklyMailMeta = G.MetaMgr.GetGame2005Mail(3)
	if this.vipLvUpMailMeta == nil || this.priviDailyMailMeta == nil || this.vipWeeklyMailMeta == nil {
		panic("game2005 mail config error")
	}
	f5.Timer().AddSimpleRepeatTimer(1000 * 10,
		func (params* q5.XParams) {
			if atomic.LoadInt64(&this.fetchEventAtomic) <= 0 {
				go this.FetchEventGo()
			} else {
				f5.SysLog().Warning("FetchEvent last pending")
			}
		})
	f5.Timer().AddSimpleFixedTimer(1000 * 10,
		func (params* q5.XParams) {
			if atomic.LoadInt64(&this.dailyMailAtomic) <= 0 {
				go this.DailyMailGo()
			} else {
				f5.SysLog().Warning("DailyMail last pending")
			}
		})
}

func (this *Game2005) UnInit() {

}

func (this *Game2005) GetGameId() int32 {
	return 2005
}

func (this *Game2005) FetchEventGo() {
	atomic.AddInt64(&this.fetchEventAtomic, 1)
	defer atomic.AddInt64(&this.fetchEventAtomic, -1)

	sendTime := int32(f5.App.NowUnix())
	for i := 0; i < this.getMysqlClusterSize(); i++ {
		conf := this.getMysqlCluster(i)
		if conf == nil {
			continue
		}
		conn := q5.NewMysql(
			conf.GetHost(),
			conf.GetPort(),
			conf.GetUser(),
			conf.GetPasswd(),
			conf.GetDatabase())
		defer conn.Close()
		if err := conn.Open(); err != nil {
			f5.SysLog().Warning(
				"FetchEventGo can't connnect %s error:%s",
				conn,
				err)
			return
		}
		if !this.FetchEventOneDB(conf, conn, sendTime) {
			f5.SysLog().Warning("FetchEventOneDB error")
			return
		}
	}
}

func (this *Game2005) DailyMailGo() {
	atomic.AddInt64(&this.dailyMailAtomic, 1)
	defer atomic.AddInt64(&this.dailyMailAtomic, -1)

	sendTime := int32(f5.App.NowUnix())
	for i := 0; i < this.getMysqlClusterSize(); i++ {
		conf := this.getMysqlCluster(i)
		if conf == nil {
			continue
		}
		conn := q5.NewMysql(
			conf.GetHost(),
			conf.GetPort(),
			conf.GetUser(),
			conf.GetPasswd(),
			conf.GetDatabase())
		defer conn.Close()
		if err := conn.Open(); err != nil {
			f5.SysLog().Warning(
				"DailyMailGo can't connnect %s error:%s",
				conn,
				err)
			return
		}
		if !this.DailyMailOneDB(conf, conn, sendTime) {
			f5.SysLog().Warning("DailyMailOneDB error")
			return
		}
	}
}

func (this *Game2005) getMysqlClusterSize() int {
	this.mysqlClusterMutex.Lock()
	defer this.mysqlClusterMutex.Unlock()
	return len(this.mysqlCluster)
}

func (this *Game2005) getMysqlCluster(index int) *MtwGame2005MysqlConf {
	this.mysqlClusterMutex.Lock()
	defer this.mysqlClusterMutex.Unlock()
	if index >= 0 && index < len(this.mysqlCluster) {
		return this.mysqlCluster[index]
	} else {
		return nil
	}
}

func (this *Game2005) GetDBIdx(instanceId int32) int64 {
	if val, ok := this.dbIdxHash.Load(instanceId); ok {
		return val.(int64)
	} else {
		return 0
	}
}

func (this *Game2005) SetDBIdx(instanceId int32, idx int64) {
	this.dbIdxHash.Store(instanceId, idx)
}

func (this *Game2005) internalSendMail(to string,
	mailSubType int32,
	content string,
	subject string,
	sendTime int32,
	expireTime int32,
	attachments string,
	uniqueKey string) bool {
	gameMailApiUrl := "https://gamemail.kingsome.cn/webapp/index.php"
	if !f5.IsOnlineEnv() {
		gameMailApiUrl = "https://gamemail-test.kingsome.cn/webapp/index.php"
	}
	params := q5.NewMxoObject()
	params.SetXValue("c", q5.NewXString("MailMgr"))
	params.SetXValue("a", q5.NewXString("sendMail"))
	params.SetXValue("gameid", q5.NewXInt32(this.GetGameId()))
	params.SetXValue("to", q5.NewXString(to))
	params.SetXValue("mailtype", q5.NewXInt32(1))
	params.SetXValue("mailsubtype", q5.NewXInt32(mailSubType))
	params.SetXValue("usertype", q5.NewXInt32(2))
	params.SetXValue("content", q5.NewXString(content))
	params.SetXValue("subject", q5.NewXString(subject))
	params.SetXValue("sendtime", q5.NewXInt32(sendTime))
	params.SetXValue("expiretime", q5.NewXInt32(expireTime))
	params.SetXValue("attachments", q5.NewXString(attachments))
	params.SetXValue("unique_key", q5.NewXString(uniqueKey))
	respObj, respStr, err := q5.HttpGetAsJson(gameMailApiUrl, params.AsXObject())
	if err != nil {
		f5.SysLog().Warning("url:%s resp:%s", gameMailApiUrl, respStr)
		return false
	}
	if !(respObj.GetSimpleStr("errcode", "") == "0" &&
		respObj.GetSimpleStr("errcode", "") == "1008") {
		f5.SysLog().Warning("url:%s resp:%s", gameMailApiUrl, respStr)
		return false
	}
	return true
}

func (this *Game2005) SendVipLvUpMail(accountId string, oldLv int32, newLv int32, upTime int32) bool {
	mailMeta := this.vipLvUpMailMeta
	for vipLv := oldLv + 1; vipLv <= newLv; vipLv++ {
		uniqueKey := fmt.Sprintf("vipLvUp_%d_%s", vipLv, accountId)
		ok := this.internalSendMail(
			accountId,
			-2,
			mailMeta.GetContent(),
			mailMeta.GetSubject(),
			upTime,
			upTime + 3600 * 24 * 30,
			mailMeta.GetAttachmentsByLv(vipLv),
			uniqueKey)
		if !ok {
			return false
		}
	}
	return true
}

func (this *Game2005) SendPriviDailyMail(accountId string, priviLv int32, sendTime int32) bool {
	mailMeta := this.priviDailyMailMeta
	uniqueKey := fmt.Sprintf("priviDaily_%s_%s", q5.FormatUnixDateEx(int64(sendTime)), accountId)
	ok := this.internalSendMail(
		accountId,
		-3,
		mailMeta.GetContent(),
		mailMeta.GetSubject(),
		sendTime,
		sendTime + 3600 * 24 * 30,
		mailMeta.GetAttachmentsByLv(priviLv),
		uniqueKey)
	return ok
}

func (this *Game2005) SendVipWeeklyMail(accountId string, vipLv int32, sendTime int32) bool {
	mailMeta := this.vipWeeklyMailMeta
	uniqueKey := fmt.Sprintf("vipWeekly_%s_%s", q5.FormatUnixDateEx(int64(sendTime)), accountId)
	ok := this.internalSendMail(
		accountId,
		-4,
		mailMeta.GetContent(),
		mailMeta.GetSubject(),
		sendTime,
		sendTime + 3600 * 24 * 30,
		mailMeta.GetAttachmentsByLv(vipLv),
		uniqueKey)
	return ok
}

func (this *Game2005) FetchEventOneDB(conf *MtwGame2005MysqlConf, conn *q5.Mysql, sendTime int32) bool {
	lastIdx := this.GetDBIdx(conf.GetInstanceId())
	for true {
		rows, err := conn.Query("SELECT idx, sender_id, event_name, param1, param2, createtime " +
			"FROM `event` " +
			"WHERE idx > ? AND status = 0 AND event_name = 'vip_level_up' " +
			"LIMIT 0, 1000;",
			lastIdx)
		if err != nil {
			f5.SysLog().Warning(
				"FetchEventOneDB queryError %s error:%s",
				conn,
				err)
			return false
		}
		defer rows.Close()
		hasData := false
		for rows.Next() {
			hasData = true
			var idx int64
			var senderId string
			var eventName string
			var param1 string
			var param2 string
			var createTime int32
			rows.Scan(&idx, &senderId, &eventName, &param1, &param2, &createTime)
			if !this.SendVipLvUpMail(
				senderId,
				q5.NewXString(param1).GetInt32(),
				q5.NewXString(param2).GetInt32(),
				createTime) {
				f5.SysLog().Warning("FetchEventOneDB sendVipLvUpMail error")
				return false
			}
			if _, err := conn.Exec("UPDATE `event` SET status = 1 WHERE idx = %d;", idx);
			err != nil {
				f5.SysLog().Warning("FetchEventOneDB updateEvent error:%s",  err)
				return false

			}
			if idx > lastIdx {
				lastIdx = idx
			}
		}
		if !hasData {
			break
		}
	}//end for
	this.SetDBIdx(conf.GetInstanceId(), lastIdx)
	return true
}

func (this *Game2005) DailyMailOneDB(conf *MtwGame2005MysqlConf, conn *q5.Mysql, sendTime int32) bool {
	procPriviDailyMail := func(accountId string,
		priviLv int32,
		nowTime int64,
		priviDailyMailLastTime int64,
		idx int64) bool {
		if priviDailyMailLastTime <= 0 ||
			q5.GetDaySeconds(nowTime) > q5.GetDaySeconds(nowTime) {
			if priviLv > 0 {
				this.SendPriviDailyMail(accountId, priviLv, sendTime)
				if _, err := conn.Exec("UPDATE `vip_user` SET privi_daily_mail_lasttime = ? WHERE idx = ?;",
					nowTime,
					idx); err != nil {
					return false

				}

			}

		}
		return true
	}
	procVipWeeklyMail := func(accountId string,
		vipLv int32,
		nowTime int64,
		vipWeeklyMailLastTime int64,
		idx int64) bool {
		if vipWeeklyMailLastTime <= 0 ||
			q5.GetDaySeconds(nowTime) > q5.GetDaySeconds(nowTime) {
			if vipLv > 0 {
				this.SendVipWeeklyMail(accountId, vipLv, sendTime)
				if _, err := conn.Exec("UPDATE `vip_user` SET vip_weekly_mail_lasttime = ? WHERE idx = ?;",
					nowTime,
					idx); err != nil {
					return false

				}
			}
		}
		return true
	}

	var lastIdx int64
	for true {
		rows, err := conn.Query("SELECT idx, accountid, vip_lv, privi_lv, createtime, modifytime, " +
			"  privi_daily_mail_lasttime, vip_weekly_mail_lasttime " +
			"FROM `vip_user` " +
			"WHERE idx > ? LIMIT 0, 1000;",
			lastIdx)
		if err != nil {
			f5.SysLog().Warning(
				"DailyMailOneDB queryError %s error:%s",
				conn,
				err)
			return false
		}
		defer rows.Close()
		hasData := false
		for rows.Next() {
			var idx int64
			var accountId string
			var vipLv int32
			var priviLv int32
			var createTime int32
			var modifyTime int32
			var priviDailyMailLastTime int64
			var vipWeeklyMailLastTime int64
			nowTime := f5.App.NowUnix()
			rows.Scan(&idx,
				&accountId,
				&vipLv,
				&priviLv,
				&createTime,
				&modifyTime,
				&priviDailyMailLastTime,
				&vipWeeklyMailLastTime)
			if !procPriviDailyMail(accountId, priviLv, nowTime, priviDailyMailLastTime, idx) {
				return false
			}
			if !procVipWeeklyMail(accountId, vipLv, nowTime, vipWeeklyMailLastTime, idx) {
				return false
			}
			if idx > lastIdx {
				lastIdx = idx
			}
		}
		if !hasData {
			break
		}
	}
	return true
}
